package com.gic.finance.service.outer.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.finance.dto.PayAccountDTO;
import com.gic.finance.entity.TabPayAccount;
import com.gic.finance.service.PayAccountApiService;
import com.gic.finance.service.PayAccountService;

@Service("payAccountApiService")
public class PayAccountApiServiceImpl implements PayAccountApiService {
    @Autowired
    private PayAccountService payAccountService;

    @Override
    public ServiceResponse<Void> save(PayAccountDTO dto) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(dto, PayAccountDTO.SavePayAccount.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        dto.setCreateTime(new Date());
        dto.setUpdateTime(new Date());
        dto.setStatus(1);
        //序号
        int sort = payAccountService.getTheMaxSort();
        dto.setSort(sort + 1);

        payAccountService.save(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> update(PayAccountDTO dto) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(dto, PayAccountDTO.SavePayAccount.class,
                PayAccountDTO.EditPayAccount.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        payAccountService.update(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> delete(Integer id) {
        TabPayAccount tab = payAccountService.getById(id);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误");
        }
        PayAccountDTO dto = new PayAccountDTO();
        dto.setPayAccountId(id);
        dto.setStatus(0);
        payAccountService.update(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> setSort(Integer id, Integer sort) {
        TabPayAccount tab = payAccountService.getById(id);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，无此记录");
        }
        if (sort == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "排序值不能为空");
        }
        payAccountService.setSort(tab, sort);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> upSort(Integer id) {
        TabPayAccount tab = payAccountService.getById(id);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误");
        }
        int minSort = payAccountService.getMinSort();
        if (minSort == tab.getSort().intValue()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已在最前面，不能上移");
        }
        TabPayAccount pre = payAccountService.getNextOfSort(tab.getSort());
        updateSortValue(id, pre.getSort());
        updateSortValue(pre.getPayAccountId(), tab.getSort());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> downSort(Integer id) {
        TabPayAccount tab = payAccountService.getById(id);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误");
        }
        int maxSort = payAccountService.getTheMaxSort();
        if (maxSort == tab.getSort().intValue()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已在最下面，不能下移");
        }
        TabPayAccount next = payAccountService.getNextOfSort(tab.getSort());
        updateSortValue(id, next.getSort());
        updateSortValue(next.getPayAccountId(), tab.getSort());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<PayAccountDTO> getById(Integer id) {
        TabPayAccount tab = payAccountService.getById(id);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(PayAccountDTO.class, tab));
    }

    @Override
    public ServiceResponse<List<PayAccountDTO>> listPayAccount() {
        List<PayAccountDTO> list = payAccountService.listPayAccount();
        if (CollectionUtils.isNotEmpty(list)) {
            for (int i = 0, len = list.size(); i < len; i++) {
                list.get(i).setSeq(i + 1);
            }
            return ServiceResponse.success(list);
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    private void updateSortValue(Integer id, Integer sort) {
        PayAccountDTO dto = new PayAccountDTO();
        dto.setPayAccountId(id);
        dto.setSort(sort);
        payAccountService.update(dto);
    }
}
